@@ -4,6 +4,7 @@
 
 import ConfigParser
 import datetime
+import json
 import os
 import socket
 import StringIO
@@ -81,7 +82,7 @@ class HTMLElement(object):
 
         :param x: X-coordinate of tap event. If not given, default to the
          center of the element.
-        :param x: X-coordinate of tap event. If not given, default to the
+        :param y: Y-coordinate of tap event. If not given, default to the
          center of the element.
         '''
         return self.marionette._send_message('singleTap', 'ok', id=self.id, x=x, y=y)
@@ -162,6 +163,18 @@ class HTMLElement(object):
 
         return self.marionette._send_message("getElementLocation", "value", id=self.id)
 
+    @property
+    def rect(self):
+        """
+            this will return a dictionary with the following:
+
+            * x and y represent the top left coordinates of the WebElement relative to top left corner of the document.
+            * height and the width will contain the height and the width of the DOMRect of the WebElement.
+
+        """
+
+        return self.marionette._send_message("getElementRect", "value", id=self.id)
+
     def value_of_css_property(self, property_name):
         '''
         Gets the value of the specified CSS property name.
@@ -450,7 +463,7 @@ class Marionette(object):
                  emulator_binary=None, emulator_res=None, connect_to_running_emulator=False,
                  gecko_log=None, homedir=None, baseurl=None, no_window=False, logdir=None,
                  busybox=None, symbols_path=None, timeout=None, socket_timeout=360,
-                 device_serial=None, adb_path=None):
+                 device_serial=None, adb_path=None, process_args=None):
         self.host = host
         self.port = self.local_port = port
         self.bin = bin
@@ -507,18 +520,20 @@ class Marionette(object):
                                             userdata=emulator_img,
                                             resolution=emulator_res,
                                             profile=profile,
-                                            adb_path=adb_path)
+                                            adb_path=adb_path,
+                                            process_args=process_args)
             self.emulator = self.runner.device
             self.emulator.start()
-            self.port = self.emulator.setup_port_forwarding(self.port)
+            self.port = self.emulator.setup_port_forwarding(remote_port=self.port)
             assert(self.emulator.wait_for_port(self.port)), "Timed out waiting for port!"
 
         if connect_to_running_emulator:
             self.runner = B2GEmulatorRunner(b2g_home=homedir,
-                                            logdir=logdir)
+                                            logdir=logdir,
+                                            process_args=process_args)
             self.emulator = self.runner.device
             self.emulator.connect()
-            self.port = self.emulator.setup_port_forwarding(self.port)
+            self.port = self.emulator.setup_port_forwarding(remote_port=self.port)
             assert(self.emulator.wait_for_port(self.port)), "Timed out waiting for port!"
 
         self.client = MarionetteTransport(self.host, self.port, self.socket_timeout)
@@ -688,6 +703,14 @@ class Marionette(object):
                 raise errors.MarionetteException(message=message, status=status, stacktrace=stacktrace)
         raise errors.MarionetteException(message=response, status=500)
 
+    def _reset_timeouts(self):
+        if self.timeout is not None:
+            self.timeouts(self.TIMEOUT_SEARCH, self.timeout)
+            self.timeouts(self.TIMEOUT_SCRIPT, self.timeout)
+            self.timeouts(self.TIMEOUT_PAGE, self.timeout)
+        else:
+            self.timeouts(self.TIMEOUT_PAGE, 30000)
+
     def check_for_crash(self):
         returncode = None
         name = None
@@ -705,6 +728,64 @@ class Marionette(object):
                 (name, returncode))
         return crashed
 
+    def enforce_gecko_prefs(self, prefs):
+        """
+        Checks if the running instance has the given prefs. If not, it will kill the
+        currently running instance, and spawn a new instance with the requested preferences.
+
+        : param prefs: A dictionary whose keys are preference names.
+        """
+        if not self.instance:
+            raise errors.MarionetteException("enforce_gecko_prefs can only be called " \
+                                             "on gecko instances launched by Marionette")
+        pref_exists = True
+        self.set_context(self.CONTEXT_CHROME)
+        for pref, value in prefs.iteritems():
+            if type(value) is not str:
+                value = json.dumps(value)
+            pref_exists = self.execute_script("""
+            let prefInterface = Components.classes["@mozilla.org/preferences-service;1"]
+                                          .getService(Components.interfaces.nsIPrefBranch);
+            let pref = '%s';
+            let value = '%s';
+            let type = prefInterface.getPrefType(pref);
+            switch(type) {
+                case prefInterface.PREF_STRING:
+                    return value == prefInterface.getCharPref(pref).toString();
+                case prefInterface.PREF_BOOL:
+                    return value == prefInterface.getBoolPref(pref).toString();
+                case prefInterface.PREF_INT:
+                    return value == prefInterface.getIntPref(pref).toString();
+                case prefInterface.PREF_INVALID:
+                    return false;
+            }
+            """ % (pref, value))
+            if not pref_exists:
+                break
+        self.set_context(self.CONTEXT_CONTENT)
+        if not pref_exists:
+            self.delete_session()
+            self.instance.restart(prefs)
+            assert(self.wait_for_port()), "Timed out waiting for port!"
+            self.start_session()
+            self._reset_timeouts()
+
+    def restart_with_clean_profile(self):
+        """
+        This will terminate the currently running instance, and spawn a new instance
+        with a clean profile.
+
+        : param prefs: A dictionary whose keys are preference names.
+        """
+        if not self.instance:
+            raise errors.MarionetteException("enforce_gecko_prefs can only be called " \
+                                             "on gecko instances launched by Marionette")
+        self.delete_session()
+        self.instance.restart()
+        assert(self.wait_for_port()), "Timed out waiting for port!"
+        self.start_session()
+        self._reset_timeouts()
+
     def absolute_url(self, relative_url):
         '''
         Returns an absolute url for files served from Marionette's www directory.
